%data section
info_length=1000000;%maximum duraion
display_step=30000; %results displaying interval
max_errors=1000;    %maximum number of errors per simulation
%diversiy-multiplexing
Nt=2;               %transmitters: must be less equal than Nr
Nrv=Nt+[0,1,2];     %receivers
%signal to noise ratio in dB
EbN0dBv=[8:1:16];
%fading
K=1; %Rice factor
correlated_fading=0;%correlated fading flag: 0 memoryless, else correlated
fdT=.001; %normalized Doppler Spread for correlated fading
%modulation
modulation_type=1; %1 for PSK and 2 for QAM
bit_per_symbol=2;
symbols=2^bit_per_symbol;
%end of data

%PSK and QAM modulation setup;
Es=[];
if modulation_type==1 %PSK
    for isym=1:symbols
        constellationI(isym)=cos(pi/symbols+isym*2*pi/symbols);
        constellationQ(isym)=sin(pi/symbols+isym*2*pi/symbols);
        constellation(isym)=constellationI(isym)+j*constellationQ(isym);
        Es(isym)=norm(constellation(isym));
    end
else %QAM
    if rem(bit_per_symbol,2)~=0
        disp('error QAM supports only even number of bit per symbol');
        return
    else
        isym=0;
        xsize=2^(bit_per_symbol-1);
        for xsym=1:xsize
            for ysym=1:xsize
                isym=isym+1;
                constellationI(isym)=2*xsym-xsize-1;
                constellationQ(isym)=2*ysym-xsize-1;
                constellation(isym)=constellationI(isym)+j*constellationQ(isym);
                Es(isym)=norm(constellation(isym));
            end
        end    
    end
end

%fading and noise setup
Pdiff=1/(1+K);
Pdir=K/(1+K);
Avenergy=mean(Es);
Nosc=16;  %for correlated fading generation

%initialization
errors=zeros(length(Nrv),length(EbN0dBv));
simulated_symbols=zeros(length(Nrv),length(EbN0dBv));
symbol_error_rate=zeros(length(Nrv),length(EbN0dBv));

%simulation
for inr=1:length(Nrv)
    Nr=Nrv(inr);
    for isnr=1:length(EbN0dBv)
        [asim,fisim,fipsim]=sosfadeli_firststep(Nosc,Nr*Nt);
        EbN0=10^(EbN0dBv(isnr)/10);
        while (errors(inr,isnr)<max_errors) && (simulated_symbols(inr,isnr)<info_length)
            simulated_symbols(inr,isnr)=simulated_symbols(inr,isnr)+1;
            info=ceil(symbols*rand(Nt,1));
        %data symbol
            infoc=constellation(info);
            infoc=reshape(infoc,Nt,1); %column vector
        %fading
            if correlated_fading
                [Hchannel,dummy,alfaI,alfaQ]=sosfadeli_onestep(simulated_symbols(inr,isnr)-1,asim,fisim,fipsim,fdT,1,K,Nosc,Nr*Nt);
                Hchannel=reshape(Hchannel,Nr,Nt);
                alfaI=reshape(alfaI,Nr,Nt);
                alfaQ=reshape(alfaQ,Nr,Nt);
            else
                alfaI=randn(Nr,Nt)*(Pdiff/2)^0.5;
                alfaQ=randn(Nr,Nt)*(Pdiff/2)^0.5+(Pdir)^0.5;
                Hchannel=(alfaI+j*alfaQ);
            end
        %noise
            sigmaN=(Avenergy*Nt*Nr/(bit_per_symbol*2*EbN0))^0.5;
            noiseI=sigmaN*randn(Nr,1);
            noiseQ=sigmaN*randn(Nr,1);
            noisec=(noiseI+j*noiseQ);
        %Vblast initialization
            rec=Hchannel*infoc+noisec;
            iVb=0;
            decidedalready=zeros(1,Nt);
        %Vblast recursion            
            while iVb<Nt
                iVb=iVb+1;
                Gchannel=pinv(Hchannel);
                for iG=1:Nt
                    testvalue(iG)=norm(Gchannel(iG,:),2); 
                end
                [dummy,kindex]=find(testvalue==min(testvalue((1-decidedalready)>0)));
                wk=Gchannel(kindex,:);
                yk=wk*rec;
                for is=1:symbols
                    distance(is)=norm(yk-constellation(is),2);
                end
                [mindist,decided]=min(distance); 
                if decided~=info(kindex);
                    errors(inr,isnr)=errors(inr,isnr)+1;
                end
                decidedalready(kindex)=1;
                rec=rec-constellation(decided)*Hchannel(:,kindex);
                Hchannel(:,kindex)=zeros(Nr,1);
            end
%periodic result display, if necessary
            if rem(simulated_symbols(inr,isnr),display_step)==0;
                symbol_error_rate(inr,isnr)=errors(inr,isnr)/(simulated_symbols(inr,isnr)*Nt);
                results=sprintf('snr=%.2f dB, errors=%4d, ser=%.2e, sim symb=%5d',EbN0dBv(isnr),errors(inr,isnr),symbol_error_rate(inr,isnr),simulated_symbols(inr,isnr));
                disp(results);
            end            
        end
%result display
        symbol_error_rate(inr,isnr)=errors(inr,isnr)/(simulated_symbols(inr,isnr)*Nt);
        results=sprintf('snr=%.2f dB, errors=%4d, ser=%.2e, Nr=%d',EbN0dBv(isnr),errors(inr,isnr),symbol_error_rate(inr,isnr),Nr);
        disp(results);
    end
end
        